package cn.com.ty.lift.business.framework.job;

import cn.com.ty.lift.business.library.dao.entity.model.response.LiftAnnualInspectionResponse;
import cn.com.ty.lift.business.library.service.LiftService;
import cn.com.ty.lift.business.maintenance.service.MaintenancePlanService;
import cn.com.ty.lift.common.model.PushMessage;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.plugins.pagination.Page;
import lombok.AllArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.springframework.jms.core.JmsMessagingTemplate;
import org.springframework.scheduling.annotation.EnableScheduling;
import org.springframework.scheduling.annotation.Scheduled;
import org.springframework.stereotype.Component;

import java.util.List;

/**
 * 定时任务
 *
 * @author wcz
 * @since 2020/1/18 17:16
 */
@Slf4j
@Component
@EnableScheduling
@AllArgsConstructor
public class BusinessJob {

    private MaintenancePlanService maintenancePlanService;
    private LiftService            liftService;
    private JmsMessagingTemplate   jmsMessagingTemplate;
    /**
     * 更新维保计划过期状态job：每天1-3点，每10分钟执行一次
     */
    @Scheduled(cron = "0 0/10 1-3 * * ?")
    public void updateMtPlanJob(){
        log.trace("Starting update maintenance Plan Job");
        maintenancePlanService.updateOverduePlan(1000);
    }

    @Scheduled(cron = "0 0 8 * * ?")
    public void inspection45days() {
        log.trace("Starting inspection 45days");
        sendAnnualInspection(45);
    }

    private void sendAnnualInspection(int days) {
        int pageNum = 1;
        long pages;
        do {
            IPage<LiftAnnualInspectionResponse> confirms = liftService.toConfirmAnnualInspection(new Page<>(pageNum, 10), 1L, days);
            pages = confirms.getPages();
            List<LiftAnnualInspectionResponse> records = confirms.getRecords();
            if (records.size() > 0) {
                if (days == 45) {
                    for (LiftAnnualInspectionResponse record : records) {
                        PushMessage.inspectionAdvance45daysNotice(record.getProjectName(), record.getDevicePosition(), record.getRegistrationCode()).sendAllOnPlatform(jmsMessagingTemplate);
                    }
                } else {
                    for (LiftAnnualInspectionResponse record : records) {
                        PushMessage.inspectionAdvance15daysNotice(record.getProjectName(), record.getDevicePosition(), record.getRegistrationCode()).sendAllOnPlatform(jmsMessagingTemplate);
                    }
                }
            }
            pageNum++;
        } while (pageNum <= pages);
    }

    @Scheduled(cron = "0 0 8 * * ?")
    public void inspection15days() {
        log.trace("Starting inspection 15days");
        sendAnnualInspection(15);
    }
}
