package cn.com.ty.lift.business.project.service;

import cn.com.ty.lift.business.capital.dao.entity.model.request.CapitalLiftRequest;
import cn.com.ty.lift.business.capital.dao.mapper.CapitalRepairLiftRelevanceMapper;
import cn.com.ty.lift.business.framework.util.MessageUtils;
import cn.com.ty.lift.business.library.dao.entity.Lift;
import cn.com.ty.lift.business.library.dao.entity.model.request.LiftExtensionRequest;
import cn.com.ty.lift.business.library.service.LiftService;
import cn.com.ty.lift.business.maintenance.dao.entity.model.request.MaintenanceAppRequest;
import cn.com.ty.lift.business.maintenance.dao.entity.model.response.MaintenancePlanResponse;
import cn.com.ty.lift.business.maintenance.dao.mapper.MaintenancePlanMapper;
import cn.com.ty.lift.business.project.dao.entity.Project;
import cn.com.ty.lift.business.project.dao.entity.model.ProjectAppUser;
import cn.com.ty.lift.business.project.dao.entity.model.request.CompanyRequest;
import cn.com.ty.lift.business.project.dao.entity.model.request.ProjectDetailRequest;
import cn.com.ty.lift.business.project.dao.entity.model.request.ProjectUserListRequest;
import cn.com.ty.lift.business.project.dao.entity.model.response.ProjectCompanyResponse;
import cn.com.ty.lift.business.project.dao.entity.model.response.ProjectDetailResponse;
import cn.com.ty.lift.business.project.dao.entity.model.response.ProjectLiftResponse;
import cn.com.ty.lift.business.project.dao.mapper.ProjectMapper;
import cn.com.ty.lift.business.project.dao.mapper.ProjectUserMapper;
import cn.com.ty.lift.common.constants.CommonEnum;
import cn.com.xwy.boot.web.dto.RestResponse;
import com.baomidou.mybatisplus.core.metadata.IPage;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

/**
 * @author bieao
 * @date 2019/12/31
 * @description app端项目管理业务层
 */
@Slf4j
@Service
public class ProjectAppService extends ServiceImpl<ProjectMapper, Project> {

    @Resource
    private LiftService liftService;

    @Resource
    private ProjectUserMapper projectUserMapper;

    @Resource
    private MaintenancePlanMapper maintenancePlanMapper;

    @Resource
    private CapitalRepairLiftRelevanceMapper capitalRepairLiftRelevanceMapper;

    /**
     * @param request 公司id
     * @return RestResponse 项目列表
     * @description 根据公司id查询公司下项目列表
     * @date 2019/12/31 4:25 PM
     */
    public IPage<ProjectCompanyResponse> list(CompanyRequest request) {
        return baseMapper.findListByCompanyId(request.getPage(), request);
    }

    /**
     * @param request 项目id、公司id
     * @return List<ProjectUser> 项目下维保工
     * @description 获取项目下维保工
     * @date 2020/1/14 10:33 上午
     */
    public List<ProjectAppUser> userList(ProjectUserListRequest request) {
        List<ProjectAppUser> userList = projectUserMapper.findProjectUserListById(request);
        return userList.stream().filter(
                entry -> (CommonEnum.DefaultRole.MTWORK.getCode().equals(entry.getUserRole())))
                .collect(Collectors.toList());
    }

    /**
     * @param request 项目id
     * @return RestResponse 电梯列表
     * @description 根据项目id查询项目下电梯列表
     * @date 2019/12/31 3:13 PM
     */
    public IPage<ProjectLiftResponse> list(ProjectDetailRequest request) {
        return baseMapper.findListByProjectId(request.getPage(), request);
    }

    /**
     * @param request 新增电梯数据项
     * @return RestResponse 状态码和返回消息
     * @description 新增电梯
     * @date 2020/1/11 4:22 下午
     */
    @Transactional(rollbackFor = Exception.class)
    public RestResponse add(LiftExtensionRequest request) {
        Lift lift = request.getLift();
        boolean result = liftService.save(lift);
        if (!result) {
            return RestResponse.fail(MessageUtils.get("msg.add.fail"));
        }
        return liftService.saveCompanyLiftInfo(lift.getId(), request.getMtCompanyId());
    }

    /**
     * @param request 项目id
     * @return RestResponse 项目详情
     * @description 根据项目id查询项目详情
     * @date 2020/1/2 1:50 下午
     */
    public ProjectDetailResponse detail(ProjectDetailRequest request) {
        return baseMapper.findProjectById(request);
    }

    /**
     * @param request 用户id和维保状态
     * @return RestResponse 保养任务列表
     * @description 根据用户id和维保状态查询保养任务列表
     * @date 2020/1/6 11:29 上午
     */
    public IPage<MaintenancePlanResponse> taskList(MaintenanceAppRequest request) {
        return maintenancePlanMapper.findListByWorkerId(request.getPage(), request);
    }

    /**
     * @param request 大修项目id
     * @return RestResponse 电梯列表
     * @description 根据大修项目id查询项目下电梯列表
     * @date 2020/4/15 4:15 下午
     */
    public IPage<ProjectLiftResponse> list(CapitalLiftRequest request) {
        return capitalRepairLiftRelevanceMapper.findListByProjectId(request.getPage(), request);
    }

    /**
     * @param request 公司id
     * @return RestResponse 项目列表
     * @description 根据公司id查询公司下大修项目列表
     * @date 2020/4/20 8:18 下午
     */
    public IPage<ProjectCompanyResponse> capitalProjectList(CompanyRequest request) {
        return capitalRepairLiftRelevanceMapper.findCapitalListByCompanyId(request.getPage(), request);
    }

}
